@@ -369,11 +369,7 @@ sm_client_xsmp_startup (EggSMClient *client,
       free (ret_client_id);
 
       gdk_threads_enter ();
-#if GTK_CHECK_VERSION (2, 24, 0)
       gdk_x11_set_sm_client_id (xsmp->client_id);
-#else
-      gdk_set_sm_client_id (xsmp->client_id);
-#endif
       gdk_threads_leave ();
 
       g_debug ("Got client ID \"%s\"", xsmp->client_id);
@@ -1158,6 +1154,7 @@ array_prop (const char *name, ...)
       pv.value = value;
       g_array_append_val (vals, pv);
     }
+  va_end (ap);
 
   prop->num_vals = vals->len;
   prop->vals = (SmPropValue *)vals->data;
