@@ -24,6 +24,7 @@
 #include "ev-document-thumbnails.h"
 #include "ev-document-links.h"
 #include "ev-document-images.h"
+#include "ev-job-scheduler.h"
 #include "ev-document-forms.h"
 #include "ev-file-exporter.h"
 #include "ev-document-factory.h"
@@ -39,6 +40,14 @@
 #include "ev-document-text.h"
 #include "ev-debug.h"
 
+#include <gtk/gtk.h>
+#if ENABLE_EPUB
+#if GTK_CHECK_VERSION(3, 0, 0)
+#include <webkit2/webkit2.h>
+#else
+#include <webkit/webkit.h>
+#endif
+#endif
 #include <errno.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
@@ -58,8 +67,8 @@ static void ev_job_page_data_init         (EvJobPageData         *job);
 static void ev_job_page_data_class_init   (EvJobPageDataClass    *class);
 static void ev_job_thumbnail_init         (EvJobThumbnail        *job);
 static void ev_job_thumbnail_class_init   (EvJobThumbnailClass   *class);
-static void ev_job_load_init    	  (EvJobLoad	         *job);
-static void ev_job_load_class_init 	  (EvJobLoadClass	 *class);
+static void ev_job_load_init              (EvJobLoad	         *job);
+static void ev_job_load_class_init 	      (EvJobLoadClass	     *class);
 static void ev_job_save_init              (EvJobSave             *job);
 static void ev_job_save_class_init        (EvJobSaveClass        *class);
 static void ev_job_find_init              (EvJobFind             *job);
@@ -87,6 +96,11 @@ enum {
 	FIND_LAST_SIGNAL
 };
 
+#if ENABLE_EPUB
+static GtkWidget* webview;
+static GtkWidget* offscreenwindow;
+#endif
+
 static guint job_signals[LAST_SIGNAL] = { 0 };
 static guint job_fonts_signals[FONTS_LAST_SIGNAL] = { 0 };
 static guint job_find_signals[FIND_LAST_SIGNAL] = { 0 };
@@ -211,13 +225,13 @@ ev_job_run (EvJob *job)
 {
 	EvJobClass *class = EV_JOB_GET_CLASS (job);
 	
-	return class->run (job);
+	return class->  run (job);
 }
 
 void
 ev_job_cancel (EvJob *job)
 {
-	if (job->cancelled || (job->finished && job->idle_finished_id == 0))
+	if (job->cancelled)
 		return;
 
 	ev_debug_message (DEBUG_JOBS, "job %s (%p) cancelled", EV_GET_TYPE_NAME (job), job);
@@ -226,6 +240,10 @@ ev_job_cancel (EvJob *job)
 	/* This should never be called from a thread */
 	job->cancelled = TRUE;
 	g_cancellable_cancel (job->cancellable);
+
+        if (job->finished && job->idle_finished_id == 0)
+                return;
+
 	g_signal_emit (job, job_signals[CANCELLED], 0);
 }
 
@@ -332,6 +350,38 @@ ev_job_links_dispose (GObject *object)
 	(* G_OBJECT_CLASS (ev_job_links_parent_class)->dispose) (object);
 }
 
+static gboolean
+fill_page_labels (GtkTreeModel   *tree_model,
+		  GtkTreePath    *path,
+		  GtkTreeIter    *iter,
+		  EvJob          *job)
+{
+	EvDocumentLinks *document_links;
+	EvLink          *link;
+	gchar           *page_label;
+
+	gtk_tree_model_get (tree_model, iter,
+			    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
+			    -1);
+
+	if (!link)
+		return FALSE;
+
+	document_links = EV_DOCUMENT_LINKS (job->document);
+	page_label = ev_document_links_get_link_page_label (document_links, link);
+	if (!page_label)
+		return FALSE;
+
+	gtk_tree_store_set (GTK_TREE_STORE (tree_model), iter,
+			    EV_DOCUMENT_LINKS_COLUMN_PAGE_LABEL, page_label,
+			    -1);
+
+	g_free (page_label);
+	g_object_unref (link);
+
+	return FALSE;
+}
+
 static gboolean
 ev_job_links_run (EvJob *job)
 {
@@ -343,7 +393,9 @@ ev_job_links_run (EvJob *job)
 	ev_document_doc_mutex_lock ();
 	job_links->model = ev_document_links_get_links_model (EV_DOCUMENT_LINKS (job->document));
 	ev_document_doc_mutex_unlock ();
-	
+
+	gtk_tree_model_foreach (job_links->model, (GtkTreeModelForeachFunc)fill_page_labels, job);
+
 	ev_job_succeeded (job);
 	
 	return FALSE;
@@ -568,6 +620,24 @@ ev_job_render_run (EvJob *job)
 	ev_document_fc_mutex_lock ();
 
 	ev_page = ev_document_get_page (job->document, job_render->page);
+
+	if ( job->document->iswebdocument == TRUE )
+	{
+		return TRUE;
+		
+		if (g_cancellable_is_cancelled (job->cancellable)) {
+		ev_document_fc_mutex_unlock ();
+		ev_document_doc_mutex_unlock ();
+		g_object_unref (rc);
+
+		return FALSE;
+		}
+		
+		ev_document_fc_mutex_unlock ();
+		ev_document_doc_mutex_unlock ();
+		ev_job_succeeded (job);
+		return FALSE;
+	}
 	rc = ev_render_context_new (ev_page, job_render->rotation, job_render->scale);
 	g_object_unref (ev_page);
 
@@ -740,7 +810,7 @@ ev_job_page_data_new (EvDocument        *document,
 static void
 ev_job_thumbnail_init (EvJobThumbnail *job)
 {
-	EV_JOB (job)->run_mode = EV_JOB_RUN_THREAD;
+		EV_JOB (job)->run_mode = EV_JOB_RUN_THREAD;
 }
 
 static void
@@ -760,28 +830,186 @@ ev_job_thumbnail_dispose (GObject *object)
 	(* G_OBJECT_CLASS (ev_job_thumbnail_parent_class)->dispose) (object);
 }
 
+#if ENABLE_EPUB
+#if !GTK_CHECK_VERSION(3, 0, 0)
+static void
+web_thumbnail_get_screenshot_cb(GObject        *object,
+                                GParamSpec     *pspec,
+                                EvJobThumbnail *job_thumb)
+{
+	WebKitWebView *webview = WEBKIT_WEB_VIEW(object);
+	WebKitLoadStatus status = webkit_web_view_get_load_status (webview);
+	if (status != WEBKIT_LOAD_FINISHED) {
+		return;
+	}
+
+	ev_document_doc_mutex_lock ();
+
+	EvPage *page = ev_document_get_page (EV_JOB(job_thumb)->document, job_thumb->page);
+	job_thumb->surface = webkit_web_view_get_snapshot (webview);
+	EvRenderContext *rc = ev_render_context_new (page, job_thumb->rotation, job_thumb->scale);
+	EvPage *screenshotpage;
+	screenshotpage = ev_page_new(job_thumb->page);
+	screenshotpage->backend_page = (EvBackendPage)job_thumb->surface;
+	screenshotpage->backend_destroy_func = (EvBackendPageDestroyFunc)cairo_surface_destroy ;
+	ev_render_context_set_page(rc,screenshotpage);
+
+	job_thumb->thumbnail = ev_document_thumbnails_get_thumbnail (EV_DOCUMENT_THUMBNAILS (EV_JOB(job_thumb)->document),
+	                                                             rc, TRUE);
+	g_object_unref(screenshotpage);
+	g_object_unref(rc);
+
+	ev_document_doc_mutex_unlock ();
+	ev_job_succeeded (EV_JOB(job_thumb));
+	return;
+}
+
+static gboolean
+webview_load_error_cb (WebKitWebView  *webview,
+                       WebKitWebFrame *web_frame,
+                       gchar          *uri,
+                       GError         *web_error,
+                       EvJobThumbnail *job_thumb)
+{
+	g_warning ("Error loading data from %s: %s", uri, web_error->message);
+	ev_job_failed_from_error (EV_JOB(job_thumb), web_error);
+	return TRUE;
+}
+#else
+static void
+snapshot_callback(WebKitWebView *webview,
+                  GAsyncResult  *results,
+                  EvJobThumbnail *job_thumb)
+{
+	GError *error = NULL;
+
+	ev_document_doc_mutex_lock ();
+
+	EvPage *page = ev_document_get_page (EV_JOB(job_thumb)->document, job_thumb->page);
+	job_thumb->surface = webkit_web_view_get_snapshot_finish (webview,
+	                                                          results,
+	                                                          &error);
+
+	if (error) {
+		g_warning ("Error retrieving a snapshot: %s", error->message);
+	}
+
+	EvRenderContext *rc = ev_render_context_new (page, job_thumb->rotation, job_thumb->scale);
+	EvPage *screenshotpage;
+	screenshotpage = ev_page_new(job_thumb->page);
+	screenshotpage->backend_page = (EvBackendPage)job_thumb->surface;
+	screenshotpage->backend_destroy_func = (EvBackendPageDestroyFunc)cairo_surface_destroy;
+	ev_render_context_set_page(rc,screenshotpage);
+
+	job_thumb->thumbnail = ev_document_thumbnails_get_thumbnail (EV_DOCUMENT_THUMBNAILS (EV_JOB(job_thumb)->document),
+	                                                             rc, TRUE);
+	g_object_unref(screenshotpage);
+	g_object_unref(rc);
+
+	ev_document_doc_mutex_unlock ();
+	ev_job_succeeded (EV_JOB(job_thumb));
+}
+
+static void
+web_thumbnail_get_screenshot_cb (WebKitWebView  *webview,
+                                 WebKitLoadEvent event,
+                                 EvJobThumbnail *job_thumb)
+{
+	if (event != WEBKIT_LOAD_FINISHED || ev_job_is_failed (EV_JOB(job_thumb))) {
+		return;
+	}
+
+	webkit_web_view_get_snapshot (webview,
+	                              WEBKIT_SNAPSHOT_REGION_VISIBLE,
+	                              WEBKIT_SNAPSHOT_OPTIONS_NONE,
+	                              NULL,
+	                              (GAsyncReadyCallback)snapshot_callback,
+	                              g_object_ref (job_thumb));
+}
+
+static gboolean
+webview_load_failed_cb (WebKitWebView  *webview,
+                        WebKitLoadEvent event,
+                        gchar          *failing_uri,
+                        gpointer        error,
+                        EvJobThumbnail *job_thumb)
+{
+	GError *e = (GError *) error;
+	g_warning ("Error loading data from %s: %s", failing_uri, e->message);
+	ev_job_failed_from_error (EV_JOB(job_thumb), e);
+	return TRUE;
+}
+#endif  /* GTK_CHECK_VERSION */
+#endif  /* ENABLE_EPUB */
+
 static gboolean
 ev_job_thumbnail_run (EvJob *job)
 {
 	EvJobThumbnail  *job_thumb = EV_JOB_THUMBNAIL (job);
 	EvRenderContext *rc;
 	EvPage          *page;
-
 	ev_debug_message (DEBUG_JOBS, "%d (%p)", job_thumb->page, job);
 	ev_profiler_start (EV_PROFILE_JOBS, "%s (%p)", EV_GET_TYPE_NAME (job), job);
-	
-	ev_document_doc_mutex_lock ();
+
+	if (job->document->iswebdocument) {
+		/* Do not block the main loop */
+		if (!ev_document_doc_mutex_trylock ())
+			return TRUE;
+	} else {
+		ev_document_doc_mutex_lock ();
+	}
 
 	page = ev_document_get_page (job->document, job_thumb->page);
-	rc = ev_render_context_new (page, job_thumb->rotation, job_thumb->scale);
-	g_object_unref (page);
-
-	job_thumb->thumbnail = ev_document_thumbnails_get_thumbnail (EV_DOCUMENT_THUMBNAILS (job->document),
-								     rc, TRUE);
-	g_object_unref (rc);
 	ev_document_doc_mutex_unlock ();
 
-	ev_job_succeeded (job);
+	if (job->document->iswebdocument == TRUE ) {
+		rc = ev_render_context_new (page, 0, job_thumb->scale);
+	} else {
+		rc = ev_render_context_new (page, job_thumb->rotation, job_thumb->scale);
+	}
+	g_object_unref (page);
+
+#if ENABLE_EPUB
+	if (job->document->iswebdocument == TRUE) {
+		if (!webview) {
+			webview = webkit_web_view_new();
+#if !GTK_CHECK_VERSION (3, 0, 0)
+			g_object_connect(WEBKIT_WEB_VIEW(webview),"signal::notify::load-status",
+			                 G_CALLBACK(web_thumbnail_get_screenshot_cb),
+			                 g_object_ref (job_thumb),
+			                 NULL);
+			g_signal_connect(WEBKIT_WEB_VIEW(webview),"load-error",
+			                 G_CALLBACK(webview_load_error_cb),
+			                 g_object_ref (job_thumb));
+#else
+			g_signal_connect(WEBKIT_WEB_VIEW(webview),"load-changed",
+			                 G_CALLBACK(web_thumbnail_get_screenshot_cb),
+			                 g_object_ref (job_thumb));
+			g_signal_connect(WEBKIT_WEB_VIEW(webview),"load-failed",
+			                 G_CALLBACK(webview_load_failed_cb),
+			                 g_object_ref (job_thumb));
+#endif  /* GTK_CHECK_VERSION */
+		}
+
+		if (!offscreenwindow) {
+			offscreenwindow = gtk_offscreen_window_new();
+			gtk_container_add(GTK_CONTAINER(offscreenwindow),GTK_WIDGET(webview));
+			gtk_window_set_default_size (GTK_WINDOW(offscreenwindow),800,1080);
+			gtk_widget_show_all(offscreenwindow);
+		}
+
+		webkit_web_view_load_uri(WEBKIT_WEB_VIEW(webview),(gchar*)rc->page->backend_page);
+	}
+	else 
+#endif  /* ENABLE_EPUB */
+	{
+		ev_document_doc_mutex_lock ();
+		job_thumb->thumbnail = ev_document_thumbnails_get_thumbnail (EV_DOCUMENT_THUMBNAILS (job->document),
+		                                                             rc, TRUE);
+		ev_document_doc_mutex_unlock ();
+		ev_job_succeeded (job);
+	}
+	g_object_unref (rc);
 	
 	return FALSE;
 }
@@ -835,8 +1063,10 @@ ev_job_fonts_run (EvJob *job)
 	if (!ev_document_doc_mutex_trylock ())
 		return TRUE;
 	
-	if (!ev_document_fc_mutex_trylock ())
+	if (!ev_document_fc_mutex_trylock ()) {
+		ev_document_doc_mutex_unlock ();
 		return TRUE;
+	}
 
 #ifdef EV_ENABLE_DEBUG
 	/* We use the #ifdef in this case because of the if */
@@ -1185,6 +1415,10 @@ ev_job_find_dispose (GObject *object)
 		g_free (job->pages);
 		job->pages = NULL;
 	}
+
+	if (job->results) {
+		g_free(job->results);
+	}
 	
 	(* G_OBJECT_CLASS (ev_job_find_parent_class)->dispose) (object);
 }
@@ -1196,7 +1430,6 @@ ev_job_find_run (EvJob *job)
 	EvDocumentFind *find = EV_DOCUMENT_FIND (job->document);
 	EvPage         *ev_page;
 	GList          *matches;
-
 	ev_debug_message (DEBUG_JOBS, NULL);
 	
 	/* Do not block the main loop */
@@ -1210,16 +1443,30 @@ ev_job_find_run (EvJob *job)
 #endif
 
 	ev_page = ev_document_get_page (job->document, job_find->current_page);
-	matches = ev_document_find_find_text (find, ev_page, job_find->text,
-					      job_find->case_sensitive);
+
+	if (job->document->iswebdocument) {
+		job_find->results[job_find->current_page] = ev_document_find_check_for_hits(find, ev_page, job_find->text,
+		                                                        job_find->case_sensitive);
+	}else {
+		matches = ev_document_find_find_text (find, ev_page, job_find->text,
+					      job_find->case_sensitive);	
+	}
+	
 	g_object_unref (ev_page);
 	
 	ev_document_doc_mutex_unlock ();
 
-	if (!job_find->has_results)
+	if (!job_find->has_results && !job->document->iswebdocument) {
 		job_find->has_results = (matches != NULL);
+	}
+	else if (!job_find->has_results && job->document->iswebdocument){
+		job_find->has_results = (job_find->results[job_find->current_page] > 0);
+	}
+
+	if (job->document->iswebdocument == FALSE) {
+		job_find->pages[job_find->current_page] = matches;
+	}
 
-	job_find->pages[job_find->current_page] = matches;
 	g_signal_emit (job_find, job_find_signals[FIND_UPDATED], 0, job_find->current_page);
 		       
 	job_find->current_page = (job_find->current_page + 1) % job_find->n_pages;
@@ -1269,7 +1516,13 @@ ev_job_find_new (EvDocument  *document,
 	job->start_page = start_page;
 	job->current_page = start_page;
 	job->n_pages = n_pages;
-	job->pages = g_new0 (GList *, n_pages);
+
+	if (document->iswebdocument) {
+		job->results = g_malloc0 (sizeof(guint) *n_pages);
+	}
+	else {
+		job->pages = g_new0 (GList *, n_pages);
+	}
 	job->text = g_strdup (text);
 	job->case_sensitive = case_sensitive;
 	job->has_results = FALSE;
@@ -1281,7 +1534,12 @@ gint
 ev_job_find_get_n_results (EvJobFind *job,
 			   gint       page)
 {
-	return g_list_length (job->pages[page]);
+	if (EV_JOB(job)->document->iswebdocument) {
+		return job->results[page];
+	}
+	else {
+		return g_list_length (job->pages[page]);
+	}
 }
 
 gdouble
@@ -1519,6 +1777,9 @@ ev_job_print_run (EvJob *job)
 
 	ev_document_doc_mutex_unlock ();
 
+        if (g_cancellable_is_cancelled (job->cancellable))
+                return FALSE;
+
 	cr_status = cairo_status (job_print->cr);
 	if (cr_status == CAIRO_STATUS_SUCCESS) {
 		ev_job_succeeded (job);
