@@ -24,9 +24,6 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#if GTK_CHECK_VERSION (3, 0, 0)
-#include <gdk/gdkkeysyms-compat.h>
-#endif
 
 #include "ev-view-presentation.h"
 #include "ev-jobs.h"
@@ -35,6 +32,10 @@
 #include "ev-view-cursor.h"
 #include "ev-page-cache.h"
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gdk_cursor_unref g_object_unref
+#endif
+
 enum {
 	PROP_0,
 	PROP_DOCUMENT,
@@ -46,6 +47,7 @@ enum {
 enum {
 	CHANGE_PAGE,
 	FINISHED,
+	SIGNAL_EXTERNAL_LINK,
 	N_SIGNALS
 };
 
@@ -60,6 +62,8 @@ struct _EvViewPresentation
 {
 	GtkWidget base;
 
+        guint                  is_constructing : 1;
+
 	guint                  current_page;
 	cairo_surface_t       *current_surface;
 	EvDocument            *document;
@@ -98,15 +102,15 @@ struct _EvViewPresentationClass
 	GtkWidgetClass base_class;
 
 	/* signals */
-	void (* change_page) (EvViewPresentation *pview,
-			      GtkScrollType       scroll);
-	void (* finished)    (EvViewPresentation *pview);
+	void (* change_page)   (EvViewPresentation *pview,
+                                GtkScrollType       scroll);
+	void (* finished)      (EvViewPresentation *pview);
+	void (* external_link) (EvViewPresentation *pview,
+                                EvLinkAction       *action);
 };
 
 static guint signals[N_SIGNALS] = { 0 };
 
-static void ev_view_presentation_next_page               (EvViewPresentation *pview);
-static void ev_view_presentation_previous_page           (EvViewPresentation *pview);
 static void ev_view_presentation_set_cursor_for_location (EvViewPresentation *pview,
 							  gdouble             x,
 							  gdouble             y);
@@ -115,18 +119,29 @@ static void ev_view_presentation_set_cursor_for_location (EvViewPresentation *pv
 
 G_DEFINE_TYPE (EvViewPresentation, ev_view_presentation, GTK_TYPE_WIDGET)
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static GdkRGBA black = { 0., 0., 0., 1. };
+static GdkRGBA white = { 1., 1., 1., 1. };
+#endif
+
 static void
 ev_view_presentation_set_normal (EvViewPresentation *pview)
 {
 	GtkWidget *widget = GTK_WIDGET (pview);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyle  *style;
+#endif
 
 	if (pview->state == EV_PRESENTATION_NORMAL)
 		return;
 
 	pview->state = EV_PRESENTATION_NORMAL;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &black);
+#else
 	style = gtk_widget_get_style (widget);
 	gdk_window_set_background (gtk_widget_get_window (widget), &style->black);
+#endif
 	gtk_widget_queue_draw (widget);
 }
 
@@ -134,14 +149,20 @@ static void
 ev_view_presentation_set_black (EvViewPresentation *pview)
 {
 	GtkWidget *widget = GTK_WIDGET (pview);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyle  *style;
+#endif
 
 	if (pview->state == EV_PRESENTATION_BLACK)
 		return;
 
 	pview->state = EV_PRESENTATION_BLACK;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &black);
+#else
 	style = gtk_widget_get_style (widget);
 	gdk_window_set_background (gtk_widget_get_window (widget), &style->black);
+#endif
 	gtk_widget_queue_draw (widget);
 }
 
@@ -149,14 +170,20 @@ static void
 ev_view_presentation_set_white (EvViewPresentation *pview)
 {
 	GtkWidget *widget = GTK_WIDGET (pview);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyle  *style;
+#endif
 
 	if (pview->state == EV_PRESENTATION_WHITE)
 		return;
 
 	pview->state = EV_PRESENTATION_WHITE;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &white);
+#else
 	style = gtk_widget_get_style (widget);
 	gdk_window_set_background (gtk_widget_get_window (widget), &style->white);
+#endif
 	gtk_widget_queue_draw (widget);
 }
 
@@ -385,6 +412,25 @@ ev_view_presentation_delete_job (EvViewPresentation *pview,
 	g_object_unref (job);
 }
 
+static void
+ev_view_presentation_reset_jobs (EvViewPresentation *pview)
+{
+        if (pview->curr_job) {
+                ev_view_presentation_delete_job (pview, pview->curr_job);
+                pview->curr_job = NULL;
+        }
+
+        if (pview->prev_job) {
+                ev_view_presentation_delete_job (pview, pview->prev_job);
+                pview->prev_job = NULL;
+        }
+
+        if (pview->next_job) {
+                ev_view_presentation_delete_job (pview, pview->next_job);
+                pview->next_job = NULL;
+        }
+}
+
 static void
 ev_view_presentation_update_current_page (EvViewPresentation *pview,
 					  guint               page)
@@ -489,7 +535,7 @@ ev_view_presentation_update_current_page (EvViewPresentation *pview,
 		gtk_widget_queue_draw (GTK_WIDGET (pview));
 }
 
-static void
+void
 ev_view_presentation_next_page (EvViewPresentation *pview)
 {
 	guint n_pages;
@@ -514,7 +560,7 @@ ev_view_presentation_next_page (EvViewPresentation *pview)
 		ev_view_presentation_update_current_page (pview, new_page);
 }
 
-static void
+void
 ev_view_presentation_previous_page (EvViewPresentation *pview)
 {
 	gint new_page = 0;
@@ -538,7 +584,7 @@ ev_view_presentation_previous_page (EvViewPresentation *pview)
 
 /* Goto Window */
 #define KEY_IS_NUMERIC(keyval) \
-	((keyval >= GDK_0 && keyval <= GDK_9) || (keyval >= GDK_KP_0 && keyval <= GDK_KP_9))
+	((keyval >= GDK_KEY_0 && keyval <= GDK_KEY_9) || (keyval >= GDK_KEY_KP_0 && keyval <= GDK_KEY_KP_9))
 
 /* Cut and paste from gtkwindow.c */
 static void
@@ -583,17 +629,17 @@ ev_view_presentation_goto_window_key_press_event (GtkWidget          *widget,
 						  EvViewPresentation *pview)
 {
 	switch (event->keyval) {
-	case GDK_Escape:
-	case GDK_Tab:
-	case GDK_KP_Tab:
-	case GDK_ISO_Left_Tab:
+	case GDK_KEY_Escape:
+	case GDK_KEY_Tab:
+	case GDK_KEY_KP_Tab:
+	case GDK_KEY_ISO_Left_Tab:
 		ev_view_presentation_goto_window_hide (pview);
 		return TRUE;
-	case GDK_Return:
-	case GDK_KP_Enter:
-	case GDK_ISO_Enter:
-	case GDK_BackSpace:
-	case GDK_Delete:
+	case GDK_KEY_Return:
+	case GDK_KEY_KP_Enter:
+	case GDK_KEY_ISO_Enter:
+	case GDK_KEY_BackSpace:
+	case GDK_KEY_Delete:
 		return FALSE;
 	default:
 		if (!KEY_IS_NUMERIC (event->keyval))
@@ -739,6 +785,9 @@ ev_view_presentation_link_is_supported (EvViewPresentation *pview,
 	case EV_LINK_ACTION_TYPE_GOTO_DEST:
 		return ev_link_action_get_dest (action) != NULL;
 	case EV_LINK_ACTION_TYPE_NAMED:
+	case EV_LINK_ACTION_TYPE_GOTO_REMOTE:
+	case EV_LINK_ACTION_TYPE_EXTERNAL_URI:
+	case EV_LINK_ACTION_TYPE_LAUNCH:
 		return TRUE;
 	default:
 		return FALSE;
@@ -797,14 +846,15 @@ ev_view_presentation_get_link_at_location (EvViewPresentation *pview,
 }
 
 static void
-ev_vew_presentation_goto_link_dest (EvViewPresentation *pview,
-				    EvLink             *link)
+ev_vew_presentation_handle_link (EvViewPresentation *pview,
+				 EvLink *link)
 {
 	EvLinkAction *action;
 
 	action = ev_link_get_action (link);
 
-	if (ev_link_action_get_action_type (action) == EV_LINK_ACTION_TYPE_NAMED) {
+	switch (ev_link_action_get_action_type (action)) {
+	case EV_LINK_ACTION_TYPE_NAMED: {
 		const gchar *name = ev_link_action_get_name (action);
 
 		if (g_ascii_strcasecmp (name, "FirstPage") == 0) {
@@ -819,7 +869,10 @@ ev_vew_presentation_goto_link_dest (EvViewPresentation *pview,
 			n_pages = ev_document_get_n_pages (pview->document);
 			ev_view_presentation_update_current_page (pview, n_pages - 1);
 		}
-	} else {
+	}
+                break;
+
+	case EV_LINK_ACTION_TYPE_GOTO_DEST: {
 		EvLinkDest *dest;
 		gint        page;
 
@@ -827,6 +880,15 @@ ev_vew_presentation_goto_link_dest (EvViewPresentation *pview,
 		page = ev_document_links_get_dest_page (EV_DOCUMENT_LINKS (pview->document), dest);
 		ev_view_presentation_update_current_page (pview, page);
 	}
+		break;
+	case EV_LINK_ACTION_TYPE_GOTO_REMOTE:
+	case EV_LINK_ACTION_TYPE_EXTERNAL_URI:
+	case EV_LINK_ACTION_TYPE_LAUNCH:
+		g_signal_emit (pview, signals[SIGNAL_EXTERNAL_LINK], 0, action);
+		break;
+	default:
+		break;
+	}
 }
 
 /* Cursors */
@@ -905,11 +967,7 @@ ev_view_presentation_update_current_surface (EvViewPresentation *pview,
 }
 
 static void
-#if GTK_CHECK_VERSION (3, 0, 0)
 ev_view_presentation_dispose (GObject *object)
-#else
-ev_view_presentation_destroy (GtkObject *object)
-#endif
 {
 	EvViewPresentation *pview = EV_VIEW_PRESENTATION (object);
 
@@ -921,21 +979,7 @@ ev_view_presentation_destroy (GtkObject *object)
 	ev_view_presentation_animation_cancel (pview);
 	ev_view_presentation_transition_stop (pview);
 	ev_view_presentation_hide_cursor_timeout_stop (pview);
-
-	if (pview->curr_job) {
-		ev_view_presentation_delete_job (pview, pview->curr_job);
-		pview->curr_job = NULL;
-	}
-
-	if (pview->prev_job) {
-		ev_view_presentation_delete_job (pview, pview->prev_job);
-		pview->prev_job = NULL;
-	}
-
-	if (pview->next_job) {
-		ev_view_presentation_delete_job (pview, pview->next_job);
-		pview->next_job = NULL;
-	}
+        ev_view_presentation_reset_jobs (pview);
 
 	if (pview->current_surface) {
 		cairo_surface_destroy (pview->current_surface);
@@ -953,11 +997,7 @@ ev_view_presentation_destroy (GtkObject *object)
 		pview->goto_entry = NULL;
 	}
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	G_OBJECT_CLASS (ev_view_presentation_parent_class)->dispose (object);
-#else
-	GTK_OBJECT_CLASS (ev_view_presentation_parent_class)->destroy (object);
-#endif
 }
 
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -1014,15 +1054,8 @@ ev_view_presentation_draw_end_page (EvViewPresentation *pview, cairo_t *cr)
 	area.height = allocation.height;
 
 #if GTK_CHECK_VERSION (3, 0, 0)
-	gtk_paint_layout (gtk_widget_get_style (widget),
-			  cr,
-			  gtk_widget_get_state (widget),
-			  FALSE,
-			  widget,
-			  NULL,
-			  15,
-			  15,
-			  layout);
+	gtk_render_layout (gtk_widget_get_style_context (widget),
+                           cr, 15, 15, layout);
 #else
 	gtk_paint_layout (gtk_widget_get_style (widget),
 			  gtk_widget_get_window (widget),
@@ -1137,38 +1170,34 @@ ev_view_presentation_key_press_event (GtkWidget   *widget,
 	EvViewPresentation *pview = EV_VIEW_PRESENTATION (widget);
 
 	if (pview->state == EV_PRESENTATION_END)
-#if GTK_CHECK_VERSION (3, 0, 0)
 		return gtk_bindings_activate_event (G_OBJECT (widget), event);
-#else
-		return gtk_bindings_activate_event (GTK_OBJECT (widget), event);
-#endif
 
 	switch (event->keyval) {
-	case GDK_b:
-	case GDK_B:
-	case GDK_period:
-	case GDK_KP_Decimal:
+	case GDK_KEY_b:
+	case GDK_KEY_B:
+	case GDK_KEY_period:
+	case GDK_KEY_KP_Decimal:
 		if (pview->state == EV_PRESENTATION_BLACK)
 			ev_view_presentation_set_normal (pview);
 		else
 			ev_view_presentation_set_black (pview);
 
 		return TRUE;
-	case GDK_w:
-	case GDK_W:
+	case GDK_KEY_w:
+	case GDK_KEY_W:
 		if (pview->state == EV_PRESENTATION_WHITE)
 			ev_view_presentation_set_normal (pview);
 		else
 			ev_view_presentation_set_white (pview);
 
 		return TRUE;
-	case GDK_Home:
+	case GDK_KEY_Home:
 		if (pview->state == EV_PRESENTATION_NORMAL) {
 			ev_view_presentation_update_current_page (pview, 0);
 			return TRUE;
 		}
 		break;
-	case GDK_End:
+	case GDK_KEY_End:
 		if (pview->state == EV_PRESENTATION_NORMAL) {
 			gint page;
 
@@ -1197,11 +1226,7 @@ ev_view_presentation_key_press_event (GtkWidget   *widget,
 		return TRUE;
 	}
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	return gtk_bindings_activate_event (G_OBJECT (widget), event);
-#else
-	return gtk_bindings_activate_event (GTK_OBJECT (widget), event);
-#endif
 }
 
 static gboolean
@@ -1224,7 +1249,7 @@ ev_view_presentation_button_release_event (GtkWidget      *widget,
 								  event->x,
 								  event->y);
 		if (link)
-			ev_vew_presentation_goto_link_dest (pview, link);
+			ev_vew_presentation_handle_link (pview, link);
 		else
 			ev_view_presentation_next_page (pview);
 	}
@@ -1286,7 +1311,9 @@ static void
 ev_view_presentation_realize (GtkWidget *widget)
 {
 	GdkWindow    *window;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkStyle     *style;
+#endif
 	GdkWindowAttr attributes;
 	GtkAllocation allocation;
 
@@ -1325,9 +1352,14 @@ ev_view_presentation_realize (GtkWidget *widget)
 	gdk_window_set_user_data (window, widget);
 	gtk_widget_set_window (widget, window);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_style_context_set_background (gtk_widget_get_style_context (widget),
+	                                  window);
+#else
 	gtk_widget_style_attach (widget);
 	style = gtk_widget_get_style (widget);
 	gdk_window_set_background (window, &style->black);
+#endif
 
 	g_idle_add ((GSourceFunc)init_presentation, widget);
 }
@@ -1380,7 +1412,7 @@ add_change_page_binding_keypad (GtkBindingSet  *binding_set,
 				GdkModifierType modifiers,
 				GtkScrollType   scroll)
 {
-	guint keypad_keyval = keyval - GDK_Left + GDK_KP_Left;
+	guint keypad_keyval = keyval - GDK_KEY_Left + GDK_KEY_KP_Left;
 
 	gtk_binding_entry_add_signal (binding_set, keyval, modifiers,
 				      "change_page", 1,
@@ -1407,7 +1439,7 @@ ev_view_presentation_set_property (GObject      *object,
 		pview->current_page = g_value_get_uint (value);
 		break;
 	case PROP_ROTATION:
-		pview->rotation = g_value_get_uint (value);
+                ev_view_presentation_set_rotation (pview, g_value_get_uint (value));
 		break;
 	case PROP_INVERTED_COLORS:
 		pview->inverted_colors = g_value_get_boolean (value);
@@ -1417,6 +1449,23 @@ ev_view_presentation_set_property (GObject      *object,
 	}
 }
 
+static void
+ev_view_presentation_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+        EvViewPresentation *pview = EV_VIEW_PRESENTATION (object);
+
+        switch (prop_id) {
+        case PROP_ROTATION:
+                g_value_set_uint (value, ev_view_presentation_get_rotation (pview));
+                break;
+        default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        }
+}
+
 static GObject *
 ev_view_presentation_constructor (GType                  type,
 				  guint                  n_construct_properties,
@@ -1429,6 +1478,7 @@ ev_view_presentation_constructor (GType                  type,
 										  n_construct_properties,
 										  construct_params);
 	pview = EV_VIEW_PRESENTATION (object);
+        pview->is_constructing = FALSE;
 
 	if (EV_IS_DOCUMENT_LINKS (pview->document)) {
 		pview->page_cache = ev_page_cache_new (pview->document);
@@ -1443,10 +1493,10 @@ ev_view_presentation_class_init (EvViewPresentationClass *klass)
 {
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GObjectClass   *gobject_class = G_OBJECT_CLASS (klass);
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
-#endif
 	GtkBindingSet  *binding_set;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCssProvider *provider;
+#endif
 
 	klass->change_page = ev_view_presentation_change_page;
 
@@ -1468,14 +1518,11 @@ ev_view_presentation_class_init (EvViewPresentationClass *klass)
 	widget_class->motion_notify_event = ev_view_presentation_motion_notify_event;
 	widget_class->scroll_event = ev_view_presentation_scroll_event;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	gobject_class->dispose = ev_view_presentation_dispose;
-#else
-	gtk_object_class->destroy = ev_view_presentation_destroy;
-#endif
 
 	gobject_class->constructor = ev_view_presentation_constructor;
 	gobject_class->set_property = ev_view_presentation_set_property;
+	gobject_class->get_property = ev_view_presentation_get_property;
 
 	g_object_class_install_property (gobject_class,
 					 PROP_DOCUMENT,
@@ -1499,8 +1546,8 @@ ev_view_presentation_class_init (EvViewPresentationClass *klass)
 							    "Rotation",
 							    "Current rotation angle",
 							    0, 360, 0,
-							    G_PARAM_WRITABLE |
-							    G_PARAM_CONSTRUCT_ONLY));
+							    G_PARAM_READWRITE |
+							    G_PARAM_CONSTRUCT));
 	g_object_class_install_property (gobject_class,
 					 PROP_INVERTED_COLORS,
 					 g_param_spec_boolean ("inverted_colors",
@@ -1528,42 +1575,64 @@ ev_view_presentation_class_init (EvViewPresentationClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0,
 			      G_TYPE_NONE);
+	signals[SIGNAL_EXTERNAL_LINK] =
+		g_signal_new ("external-link",
+			      G_TYPE_FROM_CLASS (gobject_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EvViewPresentationClass, external_link),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
 
 	binding_set = gtk_binding_set_by_class (klass);
-	add_change_page_binding_keypad (binding_set, GDK_Left,  0, GTK_SCROLL_PAGE_BACKWARD);
-	add_change_page_binding_keypad (binding_set, GDK_Right, 0, GTK_SCROLL_PAGE_FORWARD);
-	add_change_page_binding_keypad (binding_set, GDK_Up,    0, GTK_SCROLL_PAGE_BACKWARD);
-	add_change_page_binding_keypad (binding_set, GDK_Down,  0, GTK_SCROLL_PAGE_FORWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_space, 0,
+	add_change_page_binding_keypad (binding_set, GDK_KEY_Left,  0, GTK_SCROLL_PAGE_BACKWARD);
+	add_change_page_binding_keypad (binding_set, GDK_KEY_Right, 0, GTK_SCROLL_PAGE_FORWARD);
+	add_change_page_binding_keypad (binding_set, GDK_KEY_Up,    0, GTK_SCROLL_PAGE_BACKWARD);
+	add_change_page_binding_keypad (binding_set, GDK_KEY_Down,  0, GTK_SCROLL_PAGE_FORWARD);
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_space, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_BackSpace, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_BackSpace, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_Page_Down, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_Page_Down, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_Page_Up, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_Page_Up, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_J, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_J, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_H, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_H, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_L, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_L, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
-	gtk_binding_entry_add_signal (binding_set, GDK_K, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_K, 0,
 				      "change_page", 1,
 				      GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+        provider = gtk_css_provider_new ();
+        gtk_css_provider_load_from_data (provider,
+                                         "EvViewPresentation {\n"
+                                         " background-color: black; }",
+                                         -1, NULL);
+        gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                                   GTK_STYLE_PROVIDER (provider),
+                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        g_object_unref (provider);
+#endif
 }
 
 static void
 ev_view_presentation_init (EvViewPresentation *pview)
 {
 	gtk_widget_set_can_focus (GTK_WIDGET (pview), TRUE);
+        pview->is_constructing = TRUE;
 }
 
 GtkWidget *
@@ -1588,3 +1657,31 @@ ev_view_presentation_get_current_page (EvViewPresentation *pview)
 {
 	return pview->current_page;
 }
+
+void
+ev_view_presentation_set_rotation (EvViewPresentation *pview,
+                                   gint                rotation)
+{
+        if (rotation >= 360)
+                rotation -= 360;
+        else if (rotation < 0)
+                rotation += 360;
+
+        if (pview->rotation == rotation)
+                return;
+
+        pview->rotation = rotation;
+        g_object_notify (G_OBJECT (pview), "rotation");
+        if (pview->is_constructing)
+                return;
+
+        pview->scale = 0;
+        ev_view_presentation_reset_jobs (pview);
+        ev_view_presentation_update_current_page (pview, pview->current_page);
+}
+
+guint
+ev_view_presentation_get_rotation (EvViewPresentation *pview)
+{
+        return pview->rotation;
+}
